import sys
import argparse
from . import __version__
from .detector import detect_and_print

def main():
	"""
	Main entry point for the CLI
	"""
	parser = argparse.ArgumentParser(description="Raspberry Pi Hardware Info Detector")
	parser.add_argument("--version", action="version", version=f"rpi-hw-info {__version__}")
	args = parser.parse_args()
	
	return detect_and_print()

if __name__ == '__main__':
	sys.exit(main())



import subprocess
import sys

from .models import RPIModel, RPI_MODELS

def detect_rpi_model():
	"""
	Detect the Raspberry Pi model by reading /proc/cpuinfo
	
	Returns:
		RPIModel: The detected Raspberry Pi model or None if not detected
	
	Raises:
		SystemExit: If the revision format is not supported or model is not recognized
	"""
	cpuinfo = subprocess.Popen(["cat", "/proc/cpuinfo"], stdout=subprocess.PIPE)

	if not cpuinfo or not cpuinfo.stdout:
		sys.exit("Could not read /proc/cpuinfo")
	
	# see https://www.raspberrypi.org/documentation/hardware/raspberrypi/revision-codes/README.md
	revision_format_bitmask = int("0x800000", 16)
	model_id_bitmask = int("0xFF0", 16)
	
	for line in cpuinfo.stdout:
		fields = line.strip().split()
		if fields and fields[0] == b"Revision":
			revision = fields[2]
			revision_hex = int(b"0x" + revision, 16)
			
			revision_format = (revision_hex & revision_format_bitmask) >> 23
			if revision_format == 0:
				sys.exit(revision.decode() + ": older revision format `" + str(revision_format) + "' is not supported.")
			
			model_id = (revision_hex & model_id_bitmask) >> 4
			for rpi in RPI_MODELS:
				if rpi.model_id == model_id:
					return rpi
	
	if 'revision' in locals():
		sys.exit(revision.decode() + ": unrecognized revision.")
	else:
		sys.exit("Could not find Revision in /proc/cpuinfo")

def detect_and_print():
	"""
	Detect the Raspberry Pi model and print the information
	"""
	rpi_model = detect_rpi_model()
	if rpi_model:
		print(str(rpi_model))
		return 0
	return 1



class RPIModel:
	def __init__(self, model_name, model_id, cpu_target, fpu_target):
		"""Create a representation of a Raspberry Pi hardware

		Arguments:
		model_name -- the human-readable model name, e.g. "3B"
		model_id   -- Hexadecimal identifier for the RPi model; see https://www.raspberrypi.org/documentation/hardware/raspberrypi/revision-codes/README.md
		cpu_target -- the gcc cpu target to -mtune to
		fpu_target -- the gcc fpu target to -mfpu to
		"""

		self.model_name = model_name
		self.model_id = model_id
		self.cpu_target = cpu_target
		self.fpu_target = fpu_target

	def __repr__(self):
		return self.model_name + ":" + hex(self.model_id) + ":" + self.cpu_target + ":" + self.fpu_target


# List of supported Raspberry Pi models
RPI_MODELS = [
	RPIModel("3B", int("0x8", 16), "cortex-a53", "neon-fp-armv8"),
	RPIModel("3B+", int("0xd", 16), "cortex-a53", "neon-fp-armv8"),
	RPIModel("4B", int("0x11", 16), "cortex-a72", "neon-fp-armv8")
]
